==> codemirror-lang-gcode-0.1.0.tgz: package/src/lang-gcode/linter/gcode-linter.ts <==
import {gcodeSyntax} from "../gcode";
import { Diagnostic } from '@codemirror/next/lint';
import { EditorState, EditorView } from '@codemirror/next/basic-setup';

function verify(s: string, config: any): Diagnostic[] {
    const diagnostics = new Array<Diagnostic>();

    const start = new Date();
    const tree = gcodeSyntax.parser.parse(s, {strict: false});
    console.log('Parse took: ' + (new Date().getMilliseconds() - start.getMilliseconds()) + 'ms');
    // console.log(tree.toString());
    tree.iterate({
        from: 0,
        to: tree.length,
        enter: (type, start: number, end: number) => {
            // console.log(type.name, type.name.charCodeAt(0), start, end);
            // if (type.id === 0)
            if (type.name.charCodeAt(0) === 9888) {
                console.log('found error', type.name, type.name.charCodeAt(0), start, end);

                // error.severity = 'error';
                // error.from = start;
                // error.to = end;
                // error.message = 'Syntax Error: Invalid character.';
                // error.actions = [{
                //     name: "fix",
                //     apply(view: EditorView, start: number, end: number) {
                //         view.dispatch(
                //             {
                //                 changes: {
                //                     from: start,
                //                     to: end,
                //                     insert: ''
                //                 },
                //                 scrollIntoView: true
                //             })
                //     }
                // }];
                // diagnostics.push(error);
            }
        }});

    return diagnostics;
}

export function gcodeLinter(config?: any) {

    function range(state: EditorState, from= 0, to = state.doc.length) {
        return verify(state.sliceDoc(from, to), config);
    }

    return (view: EditorView) => {
        const [syntax] = view.state.facet(EditorState.syntax);
        const found: Diagnostic[] = [];
        // FIXME move to async parsing?
        syntax.getTree(view.state).iterate({
            enter(type, from, to) {
                if (type.name == gcodeSyntax.parser.topType.name) {
                    for (const diagnostic of range(view.state, from, to)) {
                        found.push(diagnostic);
                    }
                    return false;
                }
            }
        });
        return found;
    }
}

==> codemirror-lang-gcode-0.1.1.tgz: package/src/lang-gcode/linter/gcode-linter.ts <==
import {gcodeSyntax} from "../gcode";
import { EditorState, EditorView } from '@codemirror/next/basic-setup';
import { Diagnostic } from '@codemirror/next/lint';

function verify(s: string, config: any): Diagnostic[] {
    const diagnostics = new Array<Diagnostic>();

    const start = new Date();
    const tree = gcodeSyntax.parser.parse(s, {strict: false});
    console.log('Parse took: ' + (new Date().getMilliseconds() - start.getMilliseconds()) + 'ms');
    // console.log(tree.toString());
    tree.iterate({
        from: 0,
        to: tree.length,
        enter: (type, start: number, end: number) => {
            // console.log(type.name, type.name.charCodeAt(0), start, end);
            if (type.id === 0) {
            // if (type.name.charCodeAt(0) === 9888) {
                console.log('found error', type.name, type.name.charCodeAt(0), start, end);

                const error = new class implements Diagnostic {
                    from: number;
                    message: string;
                    severity: 'info' | 'warning' | 'error';
                    to: number;
                }
                error.severity = 'error';
                error.from = start;
                error.to = end;
                error.message = 'Syntax Error';
                // error.actions = [{
                //     name: "fix",
                //     apply(view: EditorView, start: number, end: number) {
                //         view.dispatch(
                //             {
                //                 changes: {
                //                     from: start,
                //                     to: end,
                //                     insert: ''
                //                 },
                //                 scrollIntoView: true
                //             })
                //     }
                // }];
                diagnostics.push(error);
            }
        }});

    return diagnostics;
}

export function gcodeLinter(config?: any) {

    function range(state: EditorState, from= 0, to = state.doc.length) {
        return verify(state.sliceDoc(from, to), config);
    }

    return (view: EditorView) => {
        const [syntax] = view.state.facet(EditorState.syntax);
        const found: Diagnostic[] = [];
        // FIXME move to async parsing?
        syntax.getTree(view.state).iterate({
            enter(type, from, to) {
                if (type.name == gcodeSyntax.parser.topType.name) {
                    for (const diagnostic of range(view.state, from, to)) {
                        found.push(diagnostic);
                    }
                    return false;
                }
            }
        });
        return found;
    }
}
